import time
from machine import Pin, I2C,PWM
from ssd1306 import SSD1306_I2C
from time import sleep_ms
from time import sleep_us

def map(x, in_min, in_max, out_min, out_max): 
    return int((x - in_min) * (out_max - out_min) /
               (in_max - in_min) + out_min)
    
class GORILLACELL_STEPMOTOR:
    def __init__(self, step_pin, dir_pin):
        self.step = Pin(step_pin, Pin.OUT)
        self.dir = Pin(dir_pin, Pin.OUT)

    def rotate(self, angle=0, rotation='cw'):
        num_of_steps = map(angle, 0, 360, 0, 200)
        if rotation=='cw':
            self.dir.value(0)
            for i in range(0,num_of_steps,1):
               self.step.value(1)
               sleep_us(500)
               self.step.value(0)
               sleep_us(500)
        if rotation=='ccw':
            self.dir.value(1)
            for i in range(num_of_steps-1,-1,-1):
               self.step.value(1)
               sleep_us(500)
               self.step.value(0)
               sleep_us(500)
               
def stepper_method(n,stepper_in):
    for round in range(52):
        if(round%n == 0):
            if(n>0):
                stepper_in.rotate(360*((n-1)/n),'ccw')
            time.sleep(2)
            #deal card
        else:
            stepper_in.rotate(360/n,'cw')
            time.sleep(2)
            #deal card
            
        

servo = PWM(Pin(32,mode=Pin.OUT))
servo.freq(50)
run_sw = Pin(15, Pin.IN ,pull=Pin.PULL_UP)
stepper = GORILLACELL_STEPMOTOR(step_pin=19, dir_pin=18)
i2c = I2C(scl=Pin(22),sda=Pin(21),freq=100000)


oled = SSD1306_I2C(128,64,i2c)
oled.fill(0)
oled.text("Enter",0,0)
oled.text("Number",0,13)
oled.text("Of",0,26)
oled.text("Players",0,39)
oled.text("(max 8 players)",0,52)
oled.show()


print("Scanning I2C bus... found",i2c.scan())
time.sleep(0.5)


print("Send data to STM32...")
i2c.writeto(0x50, b'Hello from ESP32')
time.sleep(0.5)

player_amounts = 0
show_text = '0'

while(True):
    time.sleep(0.5)
    data = i2c.readfrom(0x50,3)
    print(run_sw.value())

    if(data.decode() == "A  " and run_sw.value() == 1):
        
        oled.fill(0)
        player_amounts+= 1 
        
        if(player_amounts == 1):
            show_text = '1 player'
        elif(player_amounts == 2):
            show_text = '2 players'
        elif(player_amounts == 3):
            show_text = '3 players'
        elif(player_amounts == 4):
            show_text = '4 players'
        elif(player_amounts == 5):
            show_text = '5 players'
        elif(player_amounts == 6):
            show_text = '6 players'
        elif(player_amounts == 7):
            show_text = '7 players'
        elif(player_amounts == 8):
            show_text = '8 players'
        
        
            
        oled.text(show_text,0,0)
        oled.show()
        if(player_amounts == 8):
            player_amounts = 0
        
    elif(run_sw.value() == 0):
        stepper_method(player_amounts,stepper)

